//==============================================================================
//	File:		$URL: svn+ssh://repositorypub@repository.eecs.berkeley.edu/public/Projects/GateLib/branches/dev/Publications/Tutorials/Publications/EECS150/Labs/StructuralVerilog/Framework/FPGA_TOP_ML505.v $
//	Version:		$Revision: 26904 $
//	Author:		Greg Gibeling (http://www.gdgib.com/)
//				Chris Fletcher (http://cwfletcher.net)
//	Copyright:	Copyright 2005-2010 UC Berkeley
//==============================================================================

//==============================================================================
//	Section:	License
//==============================================================================
//	Copyright (c) 2005-2010, Regents of the University of California
//	All rights reserved.
//
//	Redistribution and use in source and binary forms, with or without modification,
//	are permitted provided that the following conditions are met:
//
//		- Redistributions of source code must retain the above copyright notice,
//			this list of conditions and the following disclaimer.
//		- Redistributions in binary form must reproduce the above copyright
//			notice, this list of conditions and the following disclaimer
//			in the documentation and/or other materials provided with the
//			distribution.
//		- Neither the name of the University of California, Berkeley nor the
//			names of its contributors may be used to endorse or promote
//			products derived from this software without specific prior
//			written permission.
//
//	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
//	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
//	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
//	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
//	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
//	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
//	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
//	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
//	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//==============================================================================

//------------------------------------------------------------------------------
//	Module:	FPGA_TOP_ML505
//	Desc:	Top level interface from a Xilinx XC5VLX50T or XC5VLX110T FPGA
//			to the ML505-50 or ML505-110 Board respectively.
//			This particular FPGA_TOP is retrofitted for the Structural Verilog
//			lab, used for CS150 (circa Spring 2009).
//	Author:	<a href="http://www.gdgib.com/">Greg Gibeling</a>
//			<a href="http://cwfletcher.net">Chris Fletcher</a>
//	Version:	$Revision: 26904 $
//------------------------------------------------------------------------------
module FPGA_TOP_ML505(
			//------------------------------------------------------------------
			//	Compass Switches & LEDs
			//------------------------------------------------------------------
			GPIO_COMPSW,	// IN(5b), W4, S3, N2, E1, C0
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	 General IO (Dipswitch, LED, Rotary Encoder & Bus Error LEDs)
			//------------------------------------------------------------------
			GPIO_DIP,
			GPIO_LED,
	
			//------------------------------------------------------------------
	
			//------------------------------------------------------------------
			//	Clocks
			//------------------------------------------------------------------
			CLK_33MHZ_FPGA,
			//------------------------------------------------------------------
		);

	//--------------------------------------------------------------------------
	//	Compass Switches & LEDs (W4, S3, N2, E1, C0)
	//--------------------------------------------------------------------------
	input	[4:0]			GPIO_COMPSW;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	 General IO (Dipswitch, LED, Rotary Encoder & Bus Error LEDs)
	//--------------------------------------------------------------------------
	input	[7:0]			GPIO_DIP;
	output	[7:0]			GPIO_LED;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Clocks
	//--------------------------------------------------------------------------
	input					CLK_33MHZ_FPGA;
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Parameters
	//		Parameters allow us to set constants throughout 
	//		our designs.  The constant 'Width' controls the
	//		width of the accumulators and ALUs that the test
	//		harness modules instantiate.
	//--------------------------------------------------------------------------
	parameter		Width =							4;						// The current width of the Accumulator/ALU in the test harness modules.
																			// IMPORTANT: If you want to modify the width of these modules, CHANGE THIS VALUE.
																			// I.E. from '4' to another constant of your choosing. (4 was an arbitrary choice)
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	User IO Wires
	//		We must declare these to be wires before they can
	//		be used.  Notice also that In and Out are 8bits
	//		wide.
	//--------------------------------------------------------------------------
	wire											ResetPin;				// Raw input signals
	wire											Clock;					// Buffered input signals
	wire			[2:0]							ALUOp;					// 3-bit ALU 'operation' signal bus
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Assigns and Decodes
	//		This section consists of "assign" statements
	//		which, in this case, are mostly to give signals 
	//		more descriptive names.
	//--------------------------------------------------------------------------
	assign			ResetPin =						GPIO_COMPSW[2];			// Reset is the north button (SW10)
	assign			ALUOp =							GPIO_DIP[2:0];			// The ALUOp is assigned using the DIP switch bank on the lower right of the board (GPIO_DIP)

    assign          GPIO_LED[7] =                   1'b0;                   // All outputs must be driven
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Clock Buffer
	//		In order to get a clean, glitch free clock signal all over the FPGA,
	//		we use this special clock buffer module.  It does not change the
	//		signal functionally, it only affects timing.
	//--------------------------------------------------------------------------
	BUFG			ClockBuf(	.I(					CLK_33MHZ_FPGA), 
								.O(					Clock));
	//--------------------------------------------------------------------------

	//--------------------------------------------------------------------------
	//	Using the Test Harnesses
	//
	//	To initiate a new test (where the Tester iterates all values through
	//	your circuit and the solution circuit), press the Reset button.  After
	//	the test completes, either the "Success" LED or the "Fail" LED will
	//	light up.  Depending on the 'Width' parameter, this might take a non-
	//	trivial amount of time.
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Accumulator Test Harness
	//--------------------------------------------------------------------------
	TestHarnessAccumulator AccHarness(.Clock(		Clock),
								.Reset(				ResetPin), 
								.ALUOp(				ALUOp),
								.Success(			GPIO_LED[0]),			// If the test succeeds, light up GPIO_LED[0]
								.Fail(				GPIO_LED[1]));			// If the test fails, light up GPIO_LED[1]
	defparam		AccHarness.Width =				Width;					// DO NOT CHANGE this line of code.
																			// If you want to change your circuit's width, change line 1065 where 'Width' is declared
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	ALU Test Harness
	//--------------------------------------------------------------------------
	TestHarnessALU 	ALUHarness(	.Clock(				Clock),
								.Reset(				ResetPin), 
								.ALUOp(				ALUOp),
								.Success(			GPIO_LED[2]),			// If the test succeeds, light up GPIO_LED[2]
								.Fail(				GPIO_LED[3]));			// If the test fails, light up GPIO_LED[3]
	defparam		ALUHarness.Width =				Width;					// DO NOT CHANGE this line of code.
																			// If you want to change your circuit's width, change line 1065 where 'Width' is declared
	//--------------------------------------------------------------------------
	
	//--------------------------------------------------------------------------
	//	Simple ALU Test Harness
	//
	//	This is a simple instantiation of the ALU, with inputs tied to DIP 
	//	switches and outputs tied to LEDs.  Use this ALU instance to 'probe'
	//	your circuit.  In other words, you expect a certain behavior from your
	//	design.  By supplying an input test vector, you can examine the output
	//	to prove correctness.
	//
	//	This is the ALU, not the Accumulator.  Also, the width is 'hard-wired'
	//	to 2 bits wide.  If you use 'generate' statements, a 2-bit implementation
	//	should accurately be able to verify an N-bit implementation.  Be sure
	//	to understand why this is the case!
	//--------------------------------------------------------------------------
	ALU 			ALUSimpleHarness(.A(			GPIO_DIP[4:3]),			// Tie the A port to 2 DIP Switches
								.B(					GPIO_DIP[6:5]),			// Tie the B port to 2 DIP Switches
								.ALUOp(				ALUOp),		
								.Result(			GPIO_LED[5:4]),		// Show the result on the LEDs
								.CarryOut(			GPIO_LED[6]));      // Show CarryOut (if any) on the LED
	defparam		ALUSimpleHarness.Width =		2;						// For simplicity's sake, we will make our simple test 2 bits wide always
																			// This is wide enough to be useful but small enough to fit the I/O on the GPIO DIP Switches and LEDs
	//--------------------------------------------------------------------------
endmodule
//------------------------------------------------------------------------------
